#ifdef TARGET_KL25Z

#include "WakeUp.h"

FunctionPointer WakeUp::callback;
float WakeUp::cycles_per_ms = 1.0;

void WakeUp::set_ms(uint32_t ms)
{
    /* Clock the timer */
    SIM->SCGC5 |= SIM_SCGC5_LPTMR_MASK;
    LPTMR0->CSR = 0;

    if (ms != 0) {
        //Clock directly from the 1kHz LPO
        LPTMR0->PSR = LPTMR_PSR_PCS(1);
        LPTMR0->PSR |= LPTMR_PSR_PBYP_MASK;

        /* Set interrupt handler */
        NVIC_SetVector(LPTimer_IRQn, (uint32_t)WakeUp::irq_handler);
        NVIC_EnableIRQ(LPTimer_IRQn);

        LPTMR0->CMR = (uint32_t)((float)ms * cycles_per_ms);

        LPTMR0->CSR = LPTMR_CSR_TIE_MASK;
        LPTMR0->CSR |= LPTMR_CSR_TEN_MASK;
    }

}


void WakeUp::irq_handler(void)
{
    // write 1 to TCF to clear the LPT timer compare flag
    LPTMR0->CSR |= LPTMR_CSR_TCF_MASK;
    callback.call();
}

void WakeUp::calibrate(void)
{
    wait_us(1);     //Otherwise next wait might overwrite our settings
    cycles_per_ms = 1.0;
    set_ms(1100);
    wait_ms(100);

    //Write first to sync value
    LPTMR0->CNR = 0;
    uint32_t ticks = LPTMR0->CNR;
    cycles_per_ms = ticks / 100.0;

    set_ms(0);
}







#endif